<?php

namespace Drupal\deprecation_status\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\deprecation_status\DataSource;

class GraphController extends ControllerBase {

  public function graphPage() {

    $build['chartjs'] = [
      '#children' => 
        '<script src="https://cdn.jsdelivr.net/npm/moment@2.24.0/moment.js"></script>
        <script src="https://cdn.jsdelivr.net/npm/chart.js@2.8.0"></script>',
    ];
    $project_graphs = [
      'top200' => 'Top 200 drupal.org contributed projects by status',
      'total' => 'All drupal.org contributed projects by status',
    ];
    foreach ($project_graphs as $graph_key => $graph_label) {
      
      // Read historic project data.
      $projects = [];
      $legend = [];
      $file = fopen(DataSource::getFullPath('breakdown_' . $graph_key . '_projects.csv'), 'r');
      $i = 0;
      while ($line = fgetcsv($file, 0, ";")) {
        if ($i == 0) {
          $legend = $line;
          array_shift($legend);
        }
        else {
          $time = array_shift($line);
          foreach($legend as $index => $label) {
            $projects[$label][$time] = $line[$index];
          }
        }
        $i++;
      }
      fclose($file);

      $breakdown_colors = [
        'Explicitly Drupal 9 compatible' => "#8CBD3F",
        'No problems found, info.yml change needed' => "#FDC70F",
        'All rector covered' => "#1F72B2",
        'Some covered by rector' => "#42509B",
        'Needs manual review' => "#EC6225",
        'No results' => "#000000",
      ];

      $datasets = [];
      foreach($projects as $label => $dataset) {
        $raw = '{label: "'. $label . '", data: [';
        foreach($dataset as $time => $value) {
          $raw .= '{x: new Date(' . $time . '*1000), y: ' . $value . '},';
        }
        $raw = trim($raw, ',') . ']';
        if (isset($breakdown_colors[$label])) {
          $raw .= ', borderColor: "' . $breakdown_colors[$label] . '", backgroundColor: "' . $breakdown_colors[$label] . '"';
        }
        $raw .= '}';
        $datasets[] = $raw;
      }
      $datasets = join(',', $datasets);

      $build[$graph_key . '_projects'] = [
        '#markup' => '<h2>' . $graph_label . '</h2><p>Projects have errors that can be <a href="https://www.drupal.org/project/rector">automatically fixed with drupal-rector</a>. Note that rector coverage and explicit Drupal 9 compatibility is only tracked on the project level since mid-March 2020.</p>',
        '#children' => <<<PROJECTSEND
  <canvas id="{$graph_key}ProjectStatus"></canvas>
  <script type="text/javascript">
  var ctx = document.getElementById('{$graph_key}ProjectStatus').getContext('2d');
  var chart = new Chart(ctx, {
    type: 'line',
    data: {
      datasets: [$datasets]
    },
    options: {
      aspectRatio: window.innerWidth < 800 ? 1 : 2,
      scales: {
        xAxes: [{
          type: 'time',
          time: {
            unit: 'week',
            max: new Date('2020-12-31')
          }
        }],
        yAxes: [{
          stacked: true,
          ticks: {
            beginAtZero: true
          }
        }]
      }
    }
  });
</script>
PROJECTSEND
      ];
    }

    // =========================================================================

    $count_graphs = [
      'top200' => 'Top 200 drupal.org contributed projects by error counts',
      'total' => 'All drupal.org contributed projects by error counts',
    ];
    foreach ($count_graphs as $graph_key => $graph_label) {
      
      // Read historic count data.
      $projects = [];
      $legend = [];
      $file = fopen(DataSource::getFullPath('breakdown_' . $graph_key . '_error_counts.csv'), 'r');
      $i = 0;
      while ($line = fgetcsv($file, 0, ";")) {
        if ($i == 0) {
          $legend = $line;
          array_shift($legend);
        }
        else {
          $time = array_shift($line);
          foreach($legend as $index => $label) {
            $projects[$label][$time] = $line[$index];
          }
        }
        $i++;
      }
      fclose($file);

      $breakdown_colors = [
        'Zero' => "#8CBD3F",
        'Less than 5' => "#FDC70F",
        'Less than 10' => "#1F72B2",
        '10 or more' => "#42509B",
        'Invalid' => "#000000",
      ];

      $datasets = [];
      foreach($projects as $label => $dataset) {
        $raw = '{label: "'. $label . '", data: [';
        foreach($dataset as $time => $value) {
          $raw .= '{x: new Date(' . $time . '*1000), y: ' . $value . '},';
        }
        $raw = trim($raw, ',') . ']';
        if (isset($breakdown_colors[$label])) {
          $raw .= ', borderColor: "' . $breakdown_colors[$label] . '", backgroundColor: "' . $breakdown_colors[$label] . '"';
        }
        $raw .= '}';
        $datasets[] = $raw;
      }
      $datasets = join(',', $datasets);

      $build[$graph_key . '_counts'] = [
        '#markup' => '<h2>' . $graph_label . '</h2><p>Number of projects by how many errors are in them.</p>',
        '#children' => <<<PROJECTSEND
  <canvas id="{$graph_key}CountStatus"></canvas>
  <script type="text/javascript">
  var ctx = document.getElementById('{$graph_key}CountStatus').getContext('2d');
  var chart = new Chart(ctx, {
    type: 'line',
    data: {
      datasets: [$datasets]
    },
    options: {
      aspectRatio: window.innerWidth < 800 ? 1 : 2,
      scales: {
        xAxes: [{
          type: 'time',
          time: {
            unit: 'week',
            max: new Date('2020-12-31')
          }
        }],
        yAxes: [{
          stacked: true,
          ticks: {
            beginAtZero: true
          }
        }]
      }
    }
  });
</script>
PROJECTSEND
      ];
    }

    // =========================================================================

    $type_graphs = [
      'top200' => 'Top 200 drupal.org contributed projects\' errors by fixability',
      'total' => 'All drupal.org contributed projects\'s errors by fixability',
    ];
    foreach ($type_graphs as $graph_key => $graph_label) {

      $legend_labels = [
        'rector' => 'Covered by rector', 
        'drupal' => 'Not covered by rector',
        'other' => 'Non-Drupal core deprecations',
        'parse' => 'Parsing errors',
      ];
      $legend_colors = [
        'Covered by rector' => "#1F72B2",
        'Not covered by rector' => "#EC6225",
        'Non-Drupal core deprecations' => "#C31A7F",
        'Parsing errors' => "#F1E70D",
      ];
      if ($graph_key == 'top200') {
        $legend_labels['archive'] = 'Historic deprecation errors';
        $legend_colors['Historic deprecation errors'] = "#CDCDCD";
      }
      $legend = array_keys($legend_labels);
      $labels = array_values($legend_labels);

      // Read historic error data.
      $errors = [];
      $file = fopen(DataSource::getFullPath('breakdown_' . $graph_key . '_error_types.csv'), 'r');
      $i = 0;
      while ($line = fgetcsv($file, 0, ";")) {
        if ($i > 0) {
          $time = array_shift($line);
          foreach($legend as $index => $label) {
            $errors[$labels[$index]][$time] = $line[$index];
          }
        }
        $i++;
      }
      fclose($file);

      $datasets = [];
      foreach($errors as $label => $dataset) {
        $raw = '{label: "'. $label . '", data: [';
        foreach($dataset as $time => $value) {
          $raw .= '{x: new Date(' . $time . '*1000), y: ' . $value . '},';
        }
        $raw = trim($raw, ',') . ']';
        if (!empty($legend_colors[$label])) {
          $raw .= ', borderColor: "' . $legend_colors[$label] . '", backgroundColor: "' . $legend_colors[$label] . '"';
        }
        $raw .= '}';
        $datasets[] = $raw;
      }
      $datasets = join(',', $datasets);

      $build[$graph_key . '_errors'] = [
        '#markup' => '<h2>' . $graph_label . '</h2><p><a href="https://www.drupal.org/project/rector">Use drupal-rector</a> to fix rector-covered errors. <a href="https://www.drupal.org/project/rector">Help expand rector coverage</a> to make all our lives easier.</p>',
        '#children' => <<<ERRORSEND
  <canvas id="{$graph_key}ErrorStatus"></canvas>
  <script type="text/javascript">
  var ctx = document.getElementById('{$graph_key}ErrorStatus').getContext('2d');
  var chart = new Chart(ctx, {
    type: 'line',
    data: {
      datasets: [$datasets]
    },
    options: {
      aspectRatio: window.innerWidth < 800 ? 1 : 2,
      scales: {
        xAxes: [{
          type: 'time',
          time: {
            unit: 'week',
            max: new Date('2020-12-31')
          }
        }],
        yAxes: [{
          stacked: true,
          ticks: {
            beginAtZero: true
          }
        }]
      }
    }
  });
</script>
ERRORSEND
      ];
    }

    // =========================================================================

    // git checkout `git rev-list -1 --before="Nov 14 2019" 9.0.x`

    // $ grep -ro "@deprecated" core | wc -l 
    // $ grep -r "@deprecated" core | grep -o "10.0.0" | wc -l
    
    // $ grep -ro "E_USER_DEPRECATED" core | wc -l
    // $ grep -r "E_USER_DEPRECATED" core | grep -o "10.0.0" | wc -l

    // Read historic core deprecation count data.
    $at_deprecated = '';
    $e_user_deprecated = '';
    $file = fopen(DataSource::getFullPath('core_deprecation_count.csv'), 'r');
    $i = 0;
    while ($line = fgetcsv($file, 0, ";")) {
      // Skip the header row.
      if ($i > 0) {
        $at_deprecated .= '{x: new Date("' . $line[0] . '"), y: ' . $line[1] . '},';
        $e_user_deprecated .= '{x: new Date("' . $line[0] . '"), y: ' . $line[2] . '},';
      }
      $i++;
    }
    fclose($file);
    $at_deprecated = trim($at_deprecated, ',');
    $e_user_deprecated = trim($e_user_deprecated, ',');

    $build['core'] = [
      '#markup' => '<h2>All deprecated APIs are removed from Drupal 9.0.x core</h2><p>Historic tracking below based on @deprecated and E_USER_DEPRECATED mentions. This is now all complete.</p>',
      '#children' => <<<COREEND
  <canvas id="coreDeprecated"></canvas>
  <script type="text/javascript">
  var ctx = document.getElementById('coreDeprecated').getContext('2d');
  var chart = new Chart(ctx, {
    type: 'line',
    data: {
      datasets: [{
        label: 'Drupal core @deprecated mentions',
        borderColor: "#FDC70F",
        borderWidth: 4,
        data: [$at_deprecated],
        cubicInterpolationMode: 'monotone',
        fill: false,
        pointRadius: 4,
        pointHoverRadius: 10,
        pointBackgroundColor: "#FDC70F",
      },
      {
        label: 'Drupal core E_USER_DEPRECATED mentions',
        borderColor: "#1F72B2",
        borderWidth: 4,
        data: [$e_user_deprecated],
        cubicInterpolationMode: 'monotone',
        fill: false,
        pointRadius: 4,
        pointHoverRadius: 10,
        pointBackgroundColor: "#1F72B2",
      }]
    },
    options: {
      scales: {
        xAxes: [{
          type: 'time',
          time: {
            unit: 'week',
            max: new Date('2020-03-02')
          }
        }],
        yAxes: [{
          ticks: {
            beginAtZero: true
          }
        }]
      }
    }
  });
</script>
COREEND
    ];

    return $build;
  }

}
